/*
 * RepMan.java
 *
 * Created for HW2 of Analysis of Software Artifacts
 * $Id: RepMan.java,v 1.5 2004/02/16 22:04:17 hisashiy Exp $
 * $Log: RepMan.java,v $
 * Revision 1.5  2004/02/16 22:04:17  hisashiy
 * Modified/added debug printing. Removed pid handling from Java code. List-mode now handles very first time access to JNDI (it now creates root node).
 *
 */

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

/**
 * RepMan class contains main method for a simple replication manager that can:
 * <ol>
 *   <li> launch programs
 *   <li> list the programs it has launched
 *   <li> kill a specified program from the programs it has launched.
 * </ol>
 *
 * @author ahmedb, hisashiy
 *
 */
public class RepMan {

    /**
     * main method handles command line and executes the command
     * in a specific mode. The options supported are:
     * <ol>
     *   <li> -p : program name and its arguments
     *   <li> -l : label to specify that command in repman
     *   <li> -m : machine name or IP address to invoke a command
     * </ol>
     * -p and -l options are required and the commandline should look
     * like:
     * <code>repman -p &lt;program &amp args&gt -l &lt;label&gt [-m &lt;machine&gt]</code>
     */
    public static void main(String[] args) {
        RepMan repman = new RepMan();
        repman.run(args);
    }

    /**
     * Runs repman. Checks the argument given and to run properly to either:
     * run, list, or kill the process.
     * @param args arguments given on the command line.
     */
    protected void run(String[] args) {
        if(args.length == 0) {
            //list the current processes
            list();
        } else if(args[0].equals("-p") || args[0].equals("-l") || args[0].equals("-m")) {
            //run command on remote (or local) host
            checkArgsAndLaunch(args);
        } else if(args[0].equals("-k")) {
            //kill command on remote (or local) host
            checkArgsAndKill(args);
        } else {
            System.out.println("Improper option specified.");
            printUsage();
            System.exit(1); //error if none of the above mode.
        }
    }

    /**
     * Lists the labels and the commands that are currently registered to
     * RepMan.
     */
    protected void list() {
        Repository repository = new Repository();
        List list = null;
        try {
            list = repository.retrieveAll();
        } catch (RepositoryException e) {
            Debug.debugPrint("Could not list commands\n"
                             + "Exception occurred: " + e);
	    //System.out.println("HIHFIDFJOD&*(&$%$T%TTGFD");
            System.exit(1);
        }

        //printing list out to System.out
        System.out.println("Listing: total # of items = ["
                           + list.size() + "]");
        Iterator i = list.iterator();
        while (i.hasNext()) {
            Command cmd = (Command)i.next();
            System.out.println(cmd.getLabel() + " " + cmd.getMachine());
        }
    }

    /**
     * Checks the argument given and sets them properly to invoke the command remotely.
     * @param args arguments to be checked and set as a command, host, and label
     */
    protected void checkArgsAndLaunch(String[] args) {
        String command = null;
        String label = null;
        String machine = null;

        //Number of arguments expected is more than or equal to 4.
        //-p option and -l option.
        if(args.length < 4){
            System.out.println("Incorrect number of arguments");
            printUsage();
            System.exit(1); //error
        }

        for(int i=0; i < args.length; i++) {
            if(args[i].equals("-l")) {
                label = args[i+1];
            } else if(args[i].equals("-m")) {
                machine = args[i+1];
            } else if(args[i].equals("-p")) {
                command = args[i+1];
            }
        }

        //check if required options are specified.
        if(label == null || command == null) {
            System.out.println("-p & -l option must be specified properly.\n");
            printUsage();
            System.exit(1);
        }

        Command cmd = new Command(command, label, machine);
        try {
            new Repository().register(cmd);
        } catch (RepositoryException e) {
            System.out.println("Could not register a command labeled: ["
			       + cmd.getLabel() + "]\n");
            Debug.debugPrint("Exception occurred: " + e);
            System.exit(1);
        }
        launch(cmd);
    }

    /**
     * Launch method uses local variables set by chechArgs method
     * @param cmd command object to be launched.
     */
    protected void launch(Command cmd) {

        String cmdLine = null;
        String result = null;

        try {
            cmdLine = cmd.createCommandLine();
            System.out.println("Launching: "+ cmdLine);
            Process p = Runtime.getRuntime().exec(cmdLine);
            //wait for exit code -- if it's 0, command worked,
            int exitCode = p.waitFor();

        } catch(IOException ex1){
            Debug.debugPrint("Could not launch cmd = '" + cmdLine + "'");
            Debug.debugPrint("Caught exception: " + ex1);
        } catch(InterruptedException ex2) {
            Debug.debugPrint("Error launching cmd='" + cmdLine + "'");
            Debug.debugPrint("Caught exception: " + ex2);
        } finally {
            //do nothing.
        }
    }

    /**
     * Checks the argument given and sets them properly to kill the labeled command.
     * @param args arguments to be checked and used to kill the labeled progam.
     */
    protected void checkArgsAndKill(String[] args) {
        String label = null;

        //Number of arguments expected is equal to 2.
        //-k option and actual argument, label.
        if(!(args.length == 2)){
            printUsage();
            System.exit(1); //error
        }

        if(args[0].equals("-k")) {
            label = args[1];
        }

        //check if required options are specified.
        if(label == null) {
            System.out.println("-k option must be specified properly.\n");
            printUsage();
            System.exit(1);
        }

        //lookup labeled commands
        Command cmd = null;
        try {
            Repository repository = new Repository();
            cmd = repository.retrieve(label);
        } catch (RepositoryException e) {
            System.out.println("Could not retrieve a command labeled: ["
                               + label + "]\n");
            Debug.debugPrint("Exception occurred: " + e);
            System.exit(1);
        }

        if(cmd != null) {
            kill(cmd);
        }
    }

    /**
     * Kills the command currently running (which is invoked previously by
     * RepMan).
     * @param cmd
     */
    protected void kill(Command cmd) {

        System.out.println("Killing: " + cmd.getLabel());
        String cmdLine = null;

        try {
            cmdLine = cmd.createKillCommandLine();
            //System.out.println("Killing Remotely: "+ cmdLine);
            Process p = Runtime.getRuntime().exec(cmdLine);

            //wait for exit code -- if it's 0, command worked,
            int exitCode = p.waitFor();
        } catch(IOException ex1){
            Debug.debugPrint("Could not launch cmd = '" + cmdLine + "'");
            Debug.debugPrint("Caught exception: " + ex1);
        } catch(InterruptedException ex2) {
            Debug.debugPrint("Error launching cmd='" + cmdLine + "'");
            Debug.debugPrint("Caught exception: " + ex2);
        } finally {
            //do nothing.
        }
    }

    /**
     * Prints out usage information to standard output.
     */
    protected void printUsage() {
        System.out.println("usage: java RepMan -p <program & " +
                           "args> -l <label> [-m <machine>]\n");
    }
}
